from holoviews.streams import PolyEdit, PolyDraw


class PolyVertexEdit(PolyEdit):
    """
    Attaches a PolyVertexEditTool and syncs the datasource.

    shared: boolean
        Whether PolyEditTools should be shared between multiple elements

    node_style: dict
        A dictionary specifying the style options for the intermediate nodes.

    feature_style: dict
        A dictionary specifying the style options for the intermediate nodes.
    """

    def __init__(self, node_style=None, feature_style=None, **params):
        if feature_style is None:
            feature_style = {}
        if node_style is None:
            node_style = {}
        self.node_style = node_style
        self.feature_style = feature_style
        super().__init__(**params)


class PolyVertexDraw(PolyDraw):
    """
    Attaches a PolyVertexDrawTool and syncs the datasource.

    shared: boolean
        Whether PolyEditTools should be shared between multiple elements

    node_style: dict
        A dictionary specifying the style options for the intermediate nodes.

    feature_style: dict
        A dictionary specifying the style options for the intermediate nodes.
    """

    def __init__(self, node_style=None, feature_style=None, **params):
        if feature_style is None:
            feature_style = {}
        if node_style is None:
            node_style = {}
        self.node_style = node_style
        self.feature_style = feature_style
        super().__init__(**params)



import param

from holoviews import (extension, help, opts, output, renderer, Store, # noqa (API import)
                       Cycle, Palette, Overlay, Layout, NdOverlay, NdLayout,
                       HoloMap, DynamicMap, GridSpace, Dimension, dim)

from holoviews import render, save # noqa (API import)

from .annotators import annotate # noqa (API import)
from .element import ( # noqa (API import)
    _Element, Feature, Tiles, WMTS, LineContours, FilledContours,
    Text, Image, ImageStack, Points, Path, Polygons, Shape, Dataset, RGB,
    Contours, Graph, TriMesh, Nodes, EdgePaths, QuadMesh, VectorField,
    HexTiles, Labels, Rectangles, Segments, WindBarbs
)
from .util import load_tiff, from_xarray # noqa (API import)
from .operation import project                      # noqa (API import)
from ._warnings import GeoviewsDeprecationWarning, GeoviewsUserWarning  # noqa: F401
from . import data                                  # noqa (API import)
from . import operation                             # noqa (API import)
from . import plotting                              # noqa (API import)
from . import feature                               # noqa (API import)
from . import tile_sources                          # noqa (API import)

__version__ = str(param.version.Version(fpath=__file__, archive_commit="$Format:%h$",
                                        reponame="geoviews"))

# Ensure opts utility is initialized with GeoViews elements
if Store._options:
    Store.set_current_backend(Store.current_backend)

# make pyct's example/data commands available if possible
from functools import partial
try:
    from pyct.cmd import copy_examples as _copy, fetch_data as _fetch, examples as _examples
    copy_examples = partial(_copy, 'geoviews')
    fetch_data = partial(_fetch, 'geoviews')
    examples = partial(_examples, 'geoviews')
except ImportError:
    def _missing_cmd(*args,**kw): return("install pyct to enable this command (e.g. `conda install -c pyviz pyct`)")
    _copy = _fetch = _examples = _missing_cmd
    def _err(): raise ValueError(_missing_cmd())
    fetch_data = copy_examples = examples = _err
del partial, _examples, _copy, _fetch



import unittest

try:
    from iris.tests.stock import lat_lon_cube
except ImportError:
    raise unittest.SkipTest("iris not available") from None

from holoviews.core import HoloMap
from holoviews.element import Curve

from geoviews.element import is_geographic, Image, Dataset
from geoviews.element.comparison import ComparisonTestCase

class TestConversions(ComparisonTestCase):

    def setUp(self):
        self.cube = lat_lon_cube()

    def test_is_geographic_2d(self):
        self.assertTrue(is_geographic(Dataset(self.cube, kdims=['longitude', 'latitude']), ['longitude', 'latitude']))

    def test_geographic_conversion(self):
        self.assertEqual(Dataset(self.cube, kdims=['longitude', 'latitude']).to.image(), Image(self.cube, kdims=['longitude', 'latitude']))

    def test_nongeographic_conversion(self):
        converted = Dataset(self.cube, kdims=['longitude', 'latitude']).to.curve(['longitude'])
        self.assertTrue(isinstance(converted, HoloMap))
        self.assertEqual(converted.kdims, ['latitude'])
        self.assertTrue(isinstance(converted.last, Curve))
